/* NAME:	GReATUtils.h
 * AUTHORS:	
 *	Attila Vizhanyo - viza@isis.vanderbilt.edu
 *
 * Various utility functions for general purposes.
 */
#ifndef __UTIL_H
#define __UTIL_H

#ifdef _MSC_VER
#pragma warning (disable : 4786) 
#pragma warning (disable : 4503)
#endif

#include <string>
#include <sstream>
#include <functional>
#include <vector>

// Returns with a string identical to 'toConvert', except with the first character upper case.
extern std::string upperBegin( const std::string& toConvert);
// Returns with a string identical to 'toConvert', except with the first character lower case.
extern std::string lowerBegin( const std::string& toConvert);
// Backslash: \ (0x5c) -> \\ 
// Question mark: ? (0x3f) -> \? 
// Single quotation mark: ' (0x27) -> \' 
// Double quotation mark: " (0x22) -> \" 
extern std::string convertEscapeSequences( const std::string& toConvert);
// Search reverse for '.', then trim 'filename' at that position, if found.
extern char* dropExtension( char* filename);
// std::string version:
extern std::string dropExtension( const std::string& filename);
// Search reverse for '.', then trim 'filename' at that position, if found.
extern std::string getExtension( const std::string& filename);
// Search reverse for '\', then trim 'filepath' at that position, if found.
extern std::string dropPath( const std::string& filepath);
// drop file name, leave path only
extern std::string getPath( const std::string& filepath);
extern std::string getPath( const std::string& filepath, bool deleteTerminatingBackSlash);
// converts name to a C++ valid name
extern std::string convertToCPlusPlusName( const std::string& name);
// converts name to a MPC (make project creator) valid name
extern std::string convertToMpcName( const std::string& name);
// Prints 'toConvert' as a hexadecimal long.
extern std::string longHexToString( long toConvert);
// tokenize comma-separated-value lines
extern int tokenize_csv_lines( const std::string& line, std::vector< std::string>& tokens);
//
extern std::string stripQuotes( const std::string& toConvert);
extern std::string encloseinQuotes( const std::string& toConvert);
extern std::string stripTrailingSlash( const std::string& toConvert);
// trim trailing '/' or '\' character, then enclose toConvert in quotes
extern std::string putPathIntoCmdLine( const std::string& toConvert);
// returns convertToCPlusPlusName( genericFuncName + "_of_" + type)
extern std::string generateCmpSpecName( const std::string& genericFuncName, const std::string& type);
// return the GReAT temp directory ('\' separated & terminated path)
extern std::string getGreatTempDir();
// return with "/* <filen> generated on <current datetime>/*"
extern std::string timestamp( const std::string& filen);
// return "$(UDM_3RDPARTY_PATH)/3rdParty/stl" for VC6. Return empty string for VC7 and later
extern std::string getStlDir();
// return with current working directory
extern std::string getCwd();
// change directory
extern void changeDir( const std::string& dir);
// Return 'path' if 'path' is absolute
// Return 'currWD'\'path' if 'path' is relative
extern std::string getFullPath( const std::string& currWD, const std::string& path);


///////////////////////////////////////////////////////////////////////////////
// returns with the first element of a pair
template< class K, class V>
K getFirst( const std::pair< K, V>& p) {
	return p.first;
}

///////////////////////////////////////////////////////////////////////////////
// delete T, then return a new instance of T
template <class T>
T* constructObj( T* def)
{
	delete def;
	return new T;
}

///////////////////////////////////////////////////////////////////////////////
// This is a different version of 'for_each', here the operand is taken by reference.
// I need it when calling ParamList's func.operator on object wrappers, and gathering some data
// in ParamList data members. As opposed to standard 'for_each', the same 'ParamList' object
// is going to be used by the iterator, not every-time copy-constructed instances.
template< class In, class Op> Op& for_each_ref( In first, In last, Op& f)
{
	while ( first!= last)
		f( *first++);
	return f;
}

///////////////////////////////////////////////////////////////////////////////
// Suitable to delete the generic pointer type value tag of ->map<- elements. 
// Not adoptable functor, but it is not intended to be.
class DeleteMapValueFunctor
{
public:
	template <class T> void operator()( const T& elem) { delete elem.second;}
};


///////////////////////////////////////////////////////////////////////////////
//	The template class defines its member function as returning t1.name() < t2.name()
template< class T>
class NameCompareFunctor : public std::binary_function< typename T, typename T, bool>
{
public:
	bool operator()( const T& t1, const T& t2) const {
		const std::string& t1Name= t1.name();
		const std::string& t2Name= t2.name();
		return t1Name < t2Name;
	}
};

///////////////////////////////////////////////////////////////////////////////
// The template returns with a string containing serialized t.
template< class T>
std::string num_to_string( T t)
{
	std::ostringstream oss;
	oss << t;
	return oss.str();
}


#ifndef INPUTFILEREGISTRY_H
///////////////////////////////////////////////////////////////////////////////
// Generic triplet class.
  template<class T1, class T2, class T3> 
  class triplet 
  {
  public:
    ///    
    typedef T1 first_type;
    ///
    typedef T2 second_type;
    ///
    typedef T3 third_type;
    ///
    triplet()
      : first(first_type()), second(second_type()), third(third_type()) {}
    ///
    triplet(const first_type& f, const second_type& s, const third_type& t)
      : first(f), second(s), third(t) {}
    ///
    first_type first;
    ///
    second_type second;
    ///
    third_type third;
  };

    ///
  template<class T1, class T2, class T3> 
  inline bool operator==(const triplet<T1,T2,T3>& x, const triplet<T1,T2,T3>& y)
  {
    return x.first == y.first && x.second == y.second && x.third == y.third;
  }
  ///
  template<class T1, class T2, class T3> 
  inline bool operator!=(const triplet<T1,T2,T3>& x, const triplet<T1,T2,T3>& y)
  {
    return !(x == y);
  }
  /// implements lexicographic ordering by using only less_than comparisions.
  template<class T1, class T2, class T3> 
  inline bool operator<(const triplet<T1,T2,T3>& x, const triplet<T1,T2,T3>& y)
  {
    return ( x.first < y.first ) || 
           ( !(y.first < x.first) && x.second < y.second ) ||
           ( !(y.first < x.first) && !(y.second < x.second) && x.third < y.third ); 
  }
  ///
  template<class T1, class T2, class T3> 
  inline bool operator>(const triplet<T1,T2,T3>& x, const triplet<T1,T2,T3>& y)
  {
    return y < x;
  }
  ///
  template<class T1, class T2, class T3> 
  inline bool operator<=(const triplet<T1,T2,T3>& x, const triplet<T1,T2,T3>& y)
  {
    return !(y < x);
  }
  ///
  template<class T1, class T2, class T3> 
  inline bool operator>=(const triplet<T1,T2,T3>& x, const triplet<T1,T2,T3>& y)
  {
    return !(x < y);
  }
  ///
  template<class T1, class T2, class T3> 
  inline triplet<T1, T2, T3> make_triplet(const T1& x, const T2& y, const T3& z)
  {
    return triplet<T1, T2, T3>(x, y, z); 
  }
#endif// INPUTFILEREGISTRY_H

#endif // __UTIL_H