/* NAME:	Config Generator
 * AUTHOR:	Attila Vizhanyo - viza@isis.vanderbilt.edu
 *
 * Generates translator & main code using the configuration information 
 * in GReATConfig models.
 */
#ifndef CONFIGGENERATOR_H
#define CONFIGGENERATOR_H

#include <map>
#include <string>
#include "GReATConfig.h"
#include "../CodeGenerator/CodeGenerator.h"
#include "DNCodePrinter.h"

#include "GReATConfigMeta.h"
#include "GReATConfigUsage.h"

// Config Generator is tied with the use of GReATConfig models.
// The code generation process of translator and main file code
// is driven by the configuration info in GReATConfing models.
class ConfigGenerator 
{
public:
	// ctor; throws CGEx if GReATCongfig model 'configFileName' has no meta info defined.
	ConfigGenerator( const std::string& configFileName) throw (CGEx);
	// dtor
	virtual ~ConfigGenerator();
	// Generates C++ code from the rule specified in the config file.
	void generate();
	// Generate the main code which opens datanetworks, and fire the rule specified in the config file.
	// Throws CGEx if GReATCongfig model has no usage info defined.
	void generateMain() throw (CGEx); 
	// Saves main program under 'mainFileName'. 
	// 'generatedRuleFileName' is the file name (without extension) under which code generator saved its output.
	void saveMain( const std::string& mainFileName, const std::string& generatedRuleFileName= "");
	// Generate User Code Library (dynamic or static)
	void generateUCL( const std::string& currDir= "", GReATConfigMeta::MetaCodeGenModes mcgm= GReATConfigMeta::MCGM_INVALID);
	//
	void generateSpecCmpCode( const std::string& currDir, GReATConfigMeta::MetaCodeGenModes mcgm);
	// generate Visual Studio workspace and project files, which can be used to build the transformation executable
	void saveVisualStudioProject( const std::string& name);
	// Generate the transformation executable
	void generateExe( const std::string& name);
	// Returns with the name of the GR transformation file used for code generation.
	std::string getTransformationFileName();
	//////////////////////////
	// CodeGenerator interface
	void generate( const GR::RootFolder& grRoot) { _cg->generate( grRoot);}
	// Generates C++ code from 'rule'.
	void generateRule( const GR::RewritingRule& rule) { _cg->generateRule( rule);}
	// Caller can gain access to the generated program, to perform printing, etc...
	const Program& getProgram() const { return _cg->getProgram();}
	// Saves the code 'name.cpp' and 'name.h' generated by 'generate()'
	void save( const std::string& name);

protected:
	// Returns with open file name, which belongs to 'fileID'
	std::string getOpenFileName( const std::string& fileID);
	// Returns with copy file name, which belongs to 'fileID'
	std::string getCopyFileName( const std::string& fileID);
	// Returns with copy file name, if not empty. Otherwise, returns with open file name.
	std::string getSaveFileName( const std::string& fileID, const std::string& mode);
	// Generate main and usage functions.
	void generateMainSkeleton( const std::vector< std::string>& fileIDs, PEComposite** tryCatchBlock);
	// Generates code which print usage & generates code which performs the default registration.
	void createUsageAndDefRegistration(
		const std::string appName, 
		const std::vector< std::string>& fileIDs, 
		PEComposite& usageComposite, 
		PEComposite& defRegComposite
		);
	// Generate code which inits/ opens/ closes datanetworks. Also, fills '_dnPrinterMap'.
	void generateDNCode( const std::vector< std::string>& fileIDs, PEComposite& addTo);
	// Iterates '_dnPrinterMap' and for add init & open code to 'addTo' for each file id.
	void createOpenDNCode( PEComposite& addTo);
	// Iterates '_dnPrinterMap' and for add close code to 'addTo' for each file id.
	void createCloseDNCode( PEComposite& addTo);
	// Generate and add definition to 'addTo' for each param in 'params'.
	void createPacketDefinitions( const ParamDatas_t& params, PEComposite& addTo);
	// Generates code (and add it to 'addTo') which traverse objects as specified in '_usageConfig'
	// and add those traversed objects to each input in 'inputs'.
	void createGetObjectsFromDNs( const ParamDatas_t& inputs, PEComposite& addTo);
	//
	void createExternGlobalDefTemporaryRootObject( PEComposite& addTo);
	void createGetTemporaryRootObject( PEComposite& addTo);
	// Generates code (and add it to 'addTo') which traverse objects recursively as specified in 'objectPath'
	// and add those traversed objects to the variable 'inputPackets'.
	// Used by createGetObjectsFromDNs()
	PEComposite*  generateObjectTraversal( 
		const std::string& parent,
//		const std::string& inputPackets,
		const std::string& meta,
		std::vector< std::string>& objectPath, 
		PEComposite* addTo,
		std::string& variableName
		);
	// Generate code (and add it to 'addTo') which instantiate class 'ruleClass', and calls its func.op.
	// with parameters 'ruleInputs' & 'ruleOutputs'.
	void createExecuteTransformation( const Class& ruleClass, const ParamDatas_t& ruleInputs, const ParamDatas_t& ruleOutputs, PEComposite& addTo);
	// Returns with the start rule specified in '_usageConfig'.
	void getStartRule( GR::RewritingRule& startRule);
	// Returns with the number of arguments the generated program will take.
	int getNoOfArgs( const std::vector< std::string>& fileIDs);
	// Generates code and add it to 'addTo', which checks attributes 'objectAttrs' for object 'objectName'.
	// Call it for the root object in an object path in '_usageConfig'.
	// Return true, if there were attribute checking code added (**checkAttr is the attribute checker PE)
	// Return false, if no attribute checking added (*checkAttr is null)
	bool generateCheckRootAttributes( 
		const std::string& objectName, 
		const std::vector< std::pair< std::string, std::string> >& objectAttrs,
		PEComposite& addTo,
		If** checkAttr
	);
	// Generates code and add it to 'addTo', which checks attributes 'objectAttrs' for object 'objectName'.
	// Call it for children objects in an object path in '_usageConfig'.
	// Return true, if there were attribute checking code added. 
	bool generateCheckChildrenAttributes( 
		const std::string& objectName, 
		const std::vector< std::pair< std::string, std::string> >& objectAttrs,
		PEComposite& addTo
	);
	// In object path, each object elem can have an arbitrary number of attributes defined.
	// All of those attributes must be checked for equality before adding the object itself to
	// the input packets of the start rule. generateCheckAttributes() generates code and add 
	// it to 'addTo', which checks attributes 'objectAttrs' for object 'objectName'.
	// Returns with the If PE which is responsible of attribute checks. (if no attributes, returns null)
	If* generateCheckAttributes( 
		const std::string& objectName, 
		const std::vector< std::pair< std::string, std::string> >& objectAttrs, // vector< attr_name, attr_value>
		PEComposite& addTo
	);
	//
	void createTempFileUsageInfos();


private:
	// Different strategies to generate cint (Guard & AM) code.
	CIntCompiler* _cic;
	CIntLibraryCall* _clc;
	// Code Generator member
	CodeGenerator* _cg;
	// Backend used to initialize CodeGenerator constructor. (See CodeGenerator.h)
	GeneralBackend _backend;
	// Config file name.
	std::string _cfg_fn;
	// Config file data network handle.
	Udm::SmartDataNetwork* _sdnConfig;
	// Main program. (The target of 'generateMain()')
	PEComposite _main;
	// Meta & usage informations of the config file.
	GReATConfig::MetaInformation _metaInfo;
	GReATConfig::UsageInformation _usageInfo;
	GReATConfigMeta* _metaConfig;
	GReATConfigUsage* _usageConfig;
	// File ID -> data network code printer object
	typedef std::map< std::string, const DNCodePrinter* > DNPrinterMap_t;
	DNPrinterMap_t _dnPrinterMap;
	// Input File Registry variable name. Set in generateMainSkeleton(), and used throughout in member functions.
	std::string _ifrName;
	// generated am and guard function header file name
	std::string _gzInc;
};

#endif //CONFIGGENERATOR_H
