#if !defined(GRDebugEvent_h)
#define GRDebugEvent_h

#include <list>

#include "GRDebugError.h"
#include "GRDebugEvent.h"
#include "GRDebugTypes.h"
 /**@defgroup HandlingEvents Handling Events
 GRDI clients can be informed of many events that occur in application programs. 
 To handle Events, designate a hook function with setEventHook. 
 For each event the hook function will be called with a Event argument describing the 
 event type and, depending on the event, additional information. 
 The hook function is usually called from within application and the GRDI 
 implementation does not queue events in any way. 
 The Event contains the event kind and a union of event-specific structures containing more information. 
 */

namespace GRDI
{
 
 
 /**
 @defgroup SingleStepEvents Single Step Events
 @ingroup HandlingEvents 
 (EVENT_SINGLE_STEP)
 Single step events allow the GRDI client to trace execution 
 at the finest granularity allowed by the GR. 
 A single step event is generated whenever a execution reaches a new location. 
 Typically, single step events represent the completion of one GRE function, i.e a Rule call.
 In any case function field of the event structure uniquely identify the current location and 
 allow  the mapping to source when that information is available. 
 */
 typedef struct 
 { 
   LocationID  m_loc;
 } SingleStepEventData;
 
 
 
 
 /**
 @defgroup BreakpointEvents Breakpoint Events
 @ingroup HandlingEvents
 (EVENT_BREAKPOINT)
 Breakpoint Events are generated whenever a thread reaches a location designated as 
 a breakpoint with setBreakpoint. The function of 
 the event structure uniquely identify the current location and allow the 
 mapping to source when that information is available. 
 */
 typedef struct 
 { 
   LocationID m_loc;
   bool       m_pause;
 } BreakpointEventData;
 
 /**
 @defgroup StackEvents Context Stack Events
 @ingroup HandlingEvents
 (EVENT_FUNCTION_ENTRY, EVENT_FUNCTION_EXIT)
 Function entry events are generated upon entry of a GRE function, i.e a GRE Rule. 
 Function exit events are generated upon exit from a GRE function, i.e a GRE Rule. 
 If a function terminates by throwing an exception,  a function exit event will not  be generated. 
 GRDI clients can check for this occurrence by monitoring exception events. 
 The m_ctxID  field provides access to the context for the function. 
 The m_func field uniquely identifies the function being entered or exited. 
 On function entry, the location reported by @ref getCurrentLocation identifies 
 the initial executable location in the function. 
 On function exit, the location reported by @ref getCurrentLocation 
 identifes the executable location in the returning function, 
 immediately prior to the return. 
 */
 typedef struct 
 { 
   LocationID   m_loc;
   ContextID    m_ctxID;
 } ContextEventData;
 
 /**
 @defgroup ExceptionEvents Exception Events
 @ingroup HandlingEvents
 (EVENT_EXCEPTION)
 Exception events are generated whenever an exception is first detected in GRE.
 The m_func field of the event structure uniquely 
 identify the current location 
 (where the exception was detected) and allow the mapping to source when that 
 information is available.
 The m_exception field identifies the thrown exception object. 
 */
 typedef struct 
 { 
   LocationID   m_loc;
 } ExceptionEventData;
 

 typedef struct 
 { 
   double   m_prgs;
   LocationID   m_loc;
 } ProgressEventData;
 

 /**
 @defgroup MiscEvent Misc Event 
 @ingroup HandlingEvents
 (EVENT_GRE_START, EVENT_GRE_STOP)
 The initialization event signals the completion of 
 GRE initialization. Once this event is generated, the GRDI client is free to call any 
 GRE or GRDI function.
 The GRE death event notifies the GRDI client of the termination of the GRE. 
 */
 
 
/// Event kind

typedef int EventKind;


#define EVENT_SINGLE_STEP        ((EventKind)1)
#define EVENT_BREAKPOINT         ((EventKind)2)
#define EVENT_EXCEPTION          ((EventKind)3)
#define EVENT_FUNCTION_ENTRY     ((EventKind)4)
#define EVENT_FUNCTION_STEP      ((EventKind)5)
#define EVENT_FUNCTION_EXIT      ((EventKind)6)
#define EVENT_CONTEXT_POP        ((EventKind)7)
#define EVENT_GR_START           ((EventKind)8)
#define EVENT_GR_FINISH          ((EventKind)9)
#define EVENT_GR_ENGINE_UP       ((EventKind)10)
#define EVENT_GR_LOAD            ((EventKind)11)
#define EVENT_GR_STOPPED         ((EventKind)12)
#define EVENT_GR_ENGINE_DOWN     ((EventKind)13)
#define EVENT_GR_SAVE            ((EventKind)14)
#define EVENT_GR_PROGRESS        ((EventKind)15)
#define EVENT_GR_CLOSE           ((EventKind)16)

#define MAX_EVENT_TYPE_VAL       ((EventKind)99)


/// This stucture passes information about the event.
/// location is the index of the last instruction executed. 
typedef struct 
{
    /// the discriminant
    EventKind m_kind;		

    union {
	      /// kind = EVENT_SINGLE_STEP
        SingleStepEventData m_singleStep;
			
        /// kind = EVENT_BREAKPOINT
        BreakpointEventData m_breakpoint;

        /// kind = EVENT_FUNCTION_ENTRY 
        /// kind = EVENT_FUNCTION_EXIT 
        ContextEventData    m_ctx;

        /// kind = EVENT_EXCEPTION
        ExceptionEventData  m_exception;
        ///kind = EVENT_GR_PROGRESS 
        ProgressEventData   m_progress;
    } m_u;
} Event;

  ///
  typedef std::list<Event> EventSet;

  ///
 class EventHook
 {
  public:
    virtual Error operator()(EventSet event) 
    {
      throw GRDException("An EventHook function is not implemented");
      return 0;
    }
 };

};



#endif// GRDebugEvent_h